// 01.11.14, Harald Peters
// Portx:  0        1        2        3        4         5         6         7
// PORTB: DIO0     DIO2     NSS      MOSI    MISO       SCK
// PortD:  RxD      TxD  FunkIn/INT0 LEDgn     LEDge                                                 
// Mega8: FuseBitsH: statt D9h C4h (EE nicht lschen, CKopt=0, Bootload D0/D1, 7680Bytes frei)
// Mega8: FuseBitsL: statt E1h BFh (14.7456Mhz, Quarz, 1:1, BOD=2.7V),    
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
//#include <avr/eeprom.h>
#include <stdarg.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>

uint8_t RFM69Spi(uint8_t Adr, uint8_t Inh) // Adr+0x80: Register schreiben
{
 PORTB &= ~(1<<PB2); // NSS = 0
 SPDR = Adr;
 while ( (SPSR&(1<<SPIF))==0 ) {}
 SPDR = Inh;
 while ( (SPSR&(1<<SPIF))==0 ) {}
 PORTB |= (1<<PB2);
 return SPDR;
}


uint16_t RFM69LiesRegPaar(uint8_t Reg) 
{
 uint16_t h = (RFM69Spi(Reg++,0xFF)<<8);
 return   h |  RFM69Spi(Reg,0xFF);
}


uint16_t RFM69SetBRate(uint16_t BRate) // in Hz
{
 uint16_t BRateH = 32E6/BRate; 
 if (BRate>0)
 {
  RFM69Spi(0x03|0x80,BRateH>>8);
  RFM69Spi(0x04|0x80,BRateH);
 }
 BRateH = RFM69LiesRegPaar(0x03);
 return 32E6/BRateH;
}


uint16_t RFM69Freq(uint16_t Freq) // 43392 -> 433.92MHz
{
 uint8_t  Reg = 0x07;
 uint32_t FreqH;
 if (Freq>0) 
 {
  FreqH = (Freq*16384UL)/100; // 2^19 = 524288
  RFM69Spi(Reg++|0x80,FreqH>>16); // immer alle 3 Reg. beschreiben!
  RFM69Spi(Reg++|0x80,FreqH>>8);
  RFM69Spi(Reg  |0x80,FreqH);
 }
 Reg = 0x07;
 FreqH =          ((uint32_t) RFM69Spi(Reg++,0xFF)) << 16;
 FreqH = FreqH + (((uint32_t) RFM69Spi(Reg++,0xFF)) <<  8);
 FreqH = FreqH +              RFM69Spi(Reg,  0xFF);

 return (FreqH  * 100UL) / 16384; // 43392 -> 433.92MHz; 2^19 = 524288
}


uint8_t RFM69Init(void) // 
{
 PORTB |=   (1<<PB2);                    // NSS=1
 DDRB  |=   (1<<PB2)|(1<<PB3)|(1<<PB5);  // SS,MOSI,SCK
 DDRB  &=  ~(1<<PB4);                    // MISO
 SPCR   =  (1<<SPE) | (1<<MSTR);         // SPI
 SPSR   =  (1<<SPI2X);                   // RFM69 ist schnell
 RFM69Freq(43392);
 RFM69Spi(0x26|0x80,0x17);     // RSSI auf DIO5; Clkout ausgeschaltet
 RFM69Spi(0x19|0x80,(1<<3)|4); // Bandbreite FSK =32MHz/(20*2^6)=25KHz
 RFM69Spi(0x6F|0x80,0x30);     // autom. AGC-Regelung
 RFM69Spi(0x01|0x80,0x10);     // Rx-Modus aktiv; 
 // Reg. 02: keine nderung: FSK, Packet-Modus
 return RFM69Spi(0x10,0xFF); // Version (0x24) zurckgeben
}


void  RFM69Scanner(void) //  RSSI
{
 uint8_t rssi;
 uint16_t Freq,FreqS;
 RFM69SetBRate(32000);

 for (Freq = 43300; Freq<43500; Freq++) 
 {
  FreqS = RFM69Freq(Freq);
  RFM69Spi(0x01|0x80,0x08); // FS-Modus
  while ( (RFM69Spi(0x27,0xFF)&0x80)==0) {;} // warten bis Modus aktiv
  RFM69Spi(0x01|0x80,0x10); // Rx-Modus
  while ( (RFM69Spi(0x27,0xFF)&0x80)==0) {;} // warten bis Modus aktiv
  
  while ( (RFM69Spi(0x23,0xFF)&0x02)==0) {;} // warten bis RSS-Messung fertig
  rssi = RFM69Spi(0x24,0xFF);  // RSSI-Wert lesen
  // jetzt FreqS und rssi ausgeben oder weiterverarbeiten
 }
}


int main(void)
{  
 wdt_disable();
 RFM69Init();
 RFM69Scanner();

 while(1)
 {


 } // while
	
 return(0);
}











